import json

from django.conf import settings
from django.forms import Media, TextInput
from wagtail.utils.widgets import WidgetWithScript


class MapInput(WidgetWithScript, TextInput):
    template_name = 'wagtailgmaps/forms/widgets/map_input.html'

    def __init__(self, default_centre, zoom, latlng, attrs=None):
        self.default_centre = default_centre
        self.zoom = zoom
        self.latlng = latlng

        try:
            self.apikey = settings.WAGTAIL_ADDRESS_MAP_KEY
        except AttributeError:
            raise Exception('Google Maps API key is missing from settings')

        super().__init__(attrs)

    def get_map_centre(self, value):
        return value or self.default_centre

    def get_map_id(self, field_id):
        assert field_id
        return '{}-map-canvas'.format(field_id)

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)

        context.update({
            'address': self.get_map_centre(value),
            'zoom': self.zoom,
            'map_id': self.get_map_id(attrs['id']),
            'gmaps_api_key': self.apikey,
        })

        return context

    def render_js_init(self, id_, name, value):
        options = {
            'map_id': self.get_map_id(id_),
            'address': self.get_map_centre(value),
            'zoom': self.zoom,
            'latlng': self.latlng,
        }
        return (
            'document.addEventListener("wagtailmaps_ready", function(e){{'
            'window.initialize_map({opts});'
            '}});'
        ).format(opts=json.dumps(options))

    @property
    def media(self):
        maps_api_js = 'https://maps.googleapis.com/maps/api/js?key={}'.format(settings.WAGTAIL_ADDRESS_MAP_KEY)
        language = getattr(settings, 'WAGTAIL_ADDRESS_MAP_LANGUAGE', None)
        if language:
            maps_api_js += '&language={}'.format(language)

        return Media(
            css={'screen': ('wagtailgmaps/css/admin.css',)},
            js=(maps_api_js, 'wagtailgmaps/js/map-field-panel.js')
        )



import unittest

from django.conf import settings
from django.test import SimpleTestCase

from wagtailgmaps.edit_handlers import MapFieldPanel
from wagtailgmaps.widgets import MapInput


class EditHandlersTestCase(SimpleTestCase):
    def test_init_with_defaults(self):
        panel = MapFieldPanel('field-name')

        self.assertEqual(panel.default_centre, settings.WAGTAIL_ADDRESS_MAP_CENTER)
        self.assertEqual(panel.zoom, 8)
        self.assertEqual(panel.latlng, False)

    def test_init_with_values(self):
        panel = MapFieldPanel(
            'field-name',
            centre='somewhere',
            zoom=0,
            latlng=True,
        )

        self.assertEqual(panel.default_centre, 'somewhere')
        self.assertEqual(panel.zoom, 0)
        self.assertEqual(panel.latlng, True)

    def test_clone(self):
        panel = MapFieldPanel(
            'field-name',
            centre='somewhere',
            zoom=0,
            latlng=True,
        )
        clone = panel.clone()

        self.assertEqual(panel.field_name, clone.field_name)
        self.assertEqual(panel.default_centre, clone.default_centre)
        self.assertEqual(panel.zoom, clone.zoom)
        self.assertEqual(panel.latlng, clone.latlng)

    @unittest.skip('TODO: Bind the panel to the model for the test to succeed.')
    def test_classes(self):
        panel = MapFieldPanel(
            'field-name',
            centre='somewhere',
            zoom=0,
            latlng=True,
        )
        classes = panel.classes()

        self.assertIn('wagtailgmap', classes)

    def test_widget_overrides(self):
        field_name = 'field-name'
        panel = MapFieldPanel(
            field_name,
            centre='somewhere',
            zoom=0,
            latlng=True,
        )
        widget_overrides = panel.widget_overrides()

        self.assertIn(field_name, widget_overrides)
        self.assertIsInstance(widget_overrides[field_name], MapInput)



from django.conf import settings
from django.test import SimpleTestCase, override_settings

from wagtailgmaps.widgets import MapInput


class MapInputTestCasse(SimpleTestCase):
    def _get_init_data(self, **kwargs):
        data = {
            'default_centre': 'Springload, Te Aro, Wellington, New Zealand',
            'zoom': 8,
            'latlng': False,
        }

        if kwargs:
            data.update(kwargs)

        return data

    def _get_widget(self):
        data = self._get_init_data()
        return MapInput(**data)

    def test_factory_is_valid(self):
        data = self._get_init_data()
        widget = MapInput(**data)

        self.assertEqual(widget.default_centre, data['default_centre'])
        self.assertEqual(widget.zoom, data['zoom'])
        self.assertEqual(widget.latlng, data['latlng'])

    @override_settings()
    def test_init_raises_for_missing_api_key(self):
        del settings.WAGTAIL_ADDRESS_MAP_KEY

        data = self._get_init_data()
        with self.assertRaises(Exception) as cm:
            MapInput(**data)

        self.assertEqual(str(cm.exception), 'Google Maps API key is missing from settings')

    def test_get_map_centre_for_none(self):
        data = self._get_init_data()
        widget = MapInput(**data)

        map_centre = widget.get_map_centre(None)
        self.assertEqual(map_centre, data['default_centre'])

    def test_get_map_centre_for_empty_value(self):
        data = self._get_init_data()
        widget = MapInput(**data)

        map_centre = widget.get_map_centre('')
        self.assertEqual(map_centre, data['default_centre'])

    def test_get_map_centre_with_value(self):
        data = self._get_init_data()
        widget = MapInput(**data)

        given_address = 'Torchbox, Charlbury, Chipping Norton, UK'
        map_centre = widget.get_map_centre(given_address)
        self.assertEqual(map_centre, given_address)

    def test_get_map_id(self):
        data = self._get_init_data()
        widget = MapInput(**data)

        field_id = 'the-field'
        expected_map_id = '{}-map-canvas'.format(field_id)
        map_id = widget.get_map_id(field_id)
        self.assertEqual(map_id, expected_map_id)

    def test_get_map_id_raises_with_no_field_id(self):
        data = self._get_init_data()
        widget = MapInput(**data)

        with self.assertRaises(AssertionError):
            widget.get_map_id(None)

        with self.assertRaises(AssertionError):
            widget.get_map_id('')

    def test_get_context(self):
        data = self._get_init_data()
        widget = MapInput(**data)
        field_id = 'the-id'

        expected_context = {
            'address': data['default_centre'],
            'zoom': data['zoom'],
            'map_id': widget.get_map_id(field_id),
            'gmaps_api_key': settings.WAGTAIL_ADDRESS_MAP_KEY,
        }
        context = widget.get_context('the-name', None, {'id': field_id})

        self.assertTrue(expected_context.items() <= context.items())

    def test_render_js_init_method_name(self):
        data = self._get_init_data()
        widget = MapInput(**data)

        js_init = widget.render_js_init('the-id', 'the-name', 'the address')
        self.assertIn('window.initialize_map', js_init)

    def test_media_css(self):
        data = self._get_init_data()
        widget = MapInput(**data)

        css = widget.media._css
        self.assertEqual(len(css), 1)
        self.assertIn('screen', css)
        self.assertEqual(len(css['screen']), 1)
        self.assertEqual(css['screen'][0], 'wagtailgmaps/css/admin.css')

    def test_media_js(self):
        data = self._get_init_data()
        widget = MapInput(**data)

        js = widget.media._js
        self.assertEqual(len(js), 2)
        self.assertEqual(
            js[0], 'https://maps.googleapis.com/maps/api/js?key={}'.format(settings.WAGTAIL_ADDRESS_MAP_KEY))
        self.assertEqual(js[1], 'wagtailgmaps/js/map-field-panel.js')

    @override_settings(WAGTAIL_ADDRESS_MAP_LANGUAGE='ru')
    def test_media_js_map_with_lang(self):
        data = self._get_init_data()
        widget = MapInput(**data)

        js = widget.media._js
        self.assertIn('&language=ru', js[0])



from django.db import models
from wagtail.admin.edit_handlers import MultiFieldPanel
from wagtail.core.models import Page

from wagtailgmaps.edit_handlers import MapFieldPanel



class MapPage(Page):
    formatted_address = models.CharField(max_length=255)
    latlng_address = models.CharField(max_length=255)

    content_panels = Page.content_panels + [
        MapFieldPanel('formatted_address'),
        MultiFieldPanel([
            MapFieldPanel('latlng_address', latlng=True),
        ], 'LatLng Address (nested)')
    ]



import json

from django.conf import settings
from wagtail.admin.edit_handlers import FieldPanel

from .widgets import MapInput


class MapFieldPanel(FieldPanel):
    def __init__(self, field_name, *args, **kwargs):
        self.default_centre = kwargs.pop('centre', getattr(settings, 'WAGTAIL_ADDRESS_MAP_CENTER', None))
        self.zoom = kwargs.pop('zoom', getattr(settings, 'WAGTAIL_ADDRESS_MAP_ZOOM', 8))
        self.latlng = kwargs.pop('latlng', False)

        super().__init__(field_name, *args, **kwargs)

    def clone(self):
        instance = super().clone()

        instance.default_centre = self.default_centre
        instance.zoom = self.zoom
        instance.latlng = self.latlng

        return instance

    def classes(self):
        classes = super().classes()
        classes.append('wagtailgmap')
        return classes

    def widget_overrides(self):
        map_input = MapInput(
            default_centre=self.default_centre,
            zoom=self.zoom,
            latlng=self.latlng,
        )
        return {self.field_name: map_input}
